using System;
using System.IO;
using UnityEngine;
using Live2D.Cubism.Core;
using Live2D.Cubism.Framework.Expression;
using Live2D.Cubism.Framework.Json;
using Live2D.Cubism.Framework.Pose;

public class main : MonoBehaviour {
	private CubismExpressionController expressionController;
	void Start(){
		var path = Application.streamingAssetsPath + "/Mao.model3.json";
		var model3Json = CubismModel3Json.LoadAtPath(path, BuiltinLoadAssetAtPath);
		var model = model3Json.ToModel(true);
		model.transform.position = new Vector3(0, 0, 1);
		model.transform.localScale = new Vector3(1000, 1000, 1);

		CubismPoseController poseController = model.gameObject.GetComponent<CubismPoseController>();
		if (model3Json.Pose3Json != null)
		{
			CubismPose3Json.SerializablePoseGroup[][] groups = model3Json.Pose3Json.Groups;
			if (groups != null)
			{
				CubismPart[] parts = model.Parts;
				for (int groupIndex=0; groupIndex<groups.Length; groupIndex++)
				{
					var group = groups[groupIndex];
					if(group == null) continue;
					for (int partIndex=0; partIndex<group.Length; partIndex++)
					{
						var part = parts.FindById(group[partIndex].Id);
						if(part == null) continue;

						var posePart = part.gameObject.GetComponent<CubismPosePart>();
						if(posePart == null)
						    posePart = part.gameObject.AddComponent<CubismPosePart>();

						posePart.GroupIndex = groupIndex;
						posePart.PartIndex = partIndex;
						posePart.Link = group[partIndex].Link;
					}
				}
			}
		}
		poseController.Refresh();
		model.ForceUpdateNow();

		CubismExp3Json[] expression3Jsons = model3Json.Expression3Jsons;
		if (expression3Jsons != null)
		{
			if (model3Json.FileReferences.Expressions != null)
			{
				CubismModel3Json.SerializableExpression[] expressions = model3Json.FileReferences.Expressions;
				expressionController = model.gameObject.GetComponent<CubismExpressionController>();
				expressionController.ExpressionsList = ScriptableObject.CreateInstance<CubismExpressionList>();
				expressionController.ExpressionsList.CubismExpressionObjects = new CubismExpressionData[expression3Jsons.Length];
				for (int i=0; i<expressions.Length; i++)
				{
					expressionController.ExpressionsList.CubismExpressionObjects[i] = CubismExpressionData.CreateInstance(expression3Jsons[i]);
				}
			}
		}
	}
	object BuiltinLoadAssetAtPath(Type assetType, string absolutePath){
		if (assetType == typeof(byte[])){
			return File.ReadAllBytes(absolutePath);
		} else if(assetType == typeof(string)){
			return File.ReadAllText(absolutePath);
		} else if (assetType == typeof(Texture2D)){
			var texture = new Texture2D(1,1);
			texture.LoadImage(File.ReadAllBytes(absolutePath));
			return texture;
		}
		throw new NotSupportedException();
	}
	void OnGUI(){
		Event e = Event.current;
		if (e == null) return;
		switch (e.type){
		case EventType.KeyUp:
			switch (e.keyCode){
			case KeyCode.UpArrow:
				if(0 < expressionController.CurrentExpressionIndex)
					expressionController.CurrentExpressionIndex--;
				break;
			case KeyCode.DownArrow:
				if(expressionController.CurrentExpressionIndex < expressionController.ExpressionsList.CubismExpressionObjects.Length-1)
					expressionController.CurrentExpressionIndex++;
				break;
			}
			break;
		}
	}
}
